﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO; //DirectoryInfo 클래스 사용
using Microsoft.Win32; //Registry 클래스 사용
using System.Runtime.InteropServices; //WindowsAPI 사용

namespace mook_Wallpaper
{
    public partial class Form1 : Form
    {
        private string file_type = "*.*"; //'img' 디렉토리의 파일 검색 타입
        private string FullPath = ""; // 파일 경로 저장
        const uint SPI_SETDESKWALLPAPAER = 0x014; //unit 타입의 상수 값 지정

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.cbPosition.Text = "가운데"; //초기 데이터 입력
            lvGetIcon(); //lvGetIcon() 메서드 호출
        }

        private void lvGetIcon()
        {
            var regKey = Registry.CurrentUser;
            regKey = regKey.CreateSubKey("Control Panel\\Desktop");
            if ((string)regKey.GetValue("Wallpaper") != "")
                this.pbImage.Image = Image.FromFile((string)regKey.GetValue("Wallpaper"));
            else
                this.pbImage.Image = null;

            var imgCount = 1;
            this.lvImage.Clear();
            this.lvImage.Items.Add(new ListViewItem(new String[] { "(없음)" }, 0));

            var dir = new DirectoryInfo(Application.StartupPath + @"\img");
            if (dir.Exists)
            {
                foreach (var f in dir.GetFiles(file_type))
                {
                    var fullname = Application.StartupPath + @"\img\" + f.Name.ToString();
                    this.ImgList.Images.Add(ExtractIcon.GetIcon(fullname, true, true));
                    this.lvImage.Items.Add(new ListViewItem(new string[] { f.Name.ToString() }, imgCount++));

                }
            }
        }

        private void lvImage_Click(object sender, EventArgs e)
        {
            if (this.lvImage.SelectedItems.Count != 0) //이미지 항목 선택시
            {
                if (this.lvImage.SelectedItems[0].SubItems[0].Text != "(없음)") // 초기 입력 이미지 선택시
                {
                    this.pbImage.Image = null; //null 이미지 개체 입력
                    FullPath = Application.StartupPath + @"\img\" + this.lvImage.SelectedItems[0].SubItems[0].Text;
                    this.pbImage.Image = Image.FromFile(Application.StartupPath + @"\img\" + this.lvImage.SelectedItems[0].SubItems[0].Text);
                    this.cbPosition.Enabled = true; //cbPosition [Enabled] 속성값 활성화
                }
                else
                {
                    this.cbPosition.Enabled = false;
                    this.pbImage.Image = null;
                }
            }
        }

        private void btnFile_Click(object sender, EventArgs e)
        {
            this.ofdFile.Filter = "이미지(*.bmp)|*.bmp|(*.gif)|*.gif|(*.jpg)|*.jpg"; //[열기] 대화상자에서 이미지만 열람가능하게
            if (this.ofdFile.ShowDialog() == DialogResult.OK) // 이미지 파일 선택시
            {
                FullPath = this.ofdFile.FileName;
                this.pbImage.Image = Image.FromFile(this.ofdFile.FileName); //지정된 경로에 따라 이미지 pbImage 컨트롤에 출력
                if (this.pbImage.Image != null)
                    this.cbPosition.Enabled = true; //cbPosition [Enabled] 속성값 활성화
            }
        }

        [DllImport("user32.dll")]
        public static extern bool SystemParametersInfo(
            uint uiAction, uint uiParam, String pvParam, uint fWinIni);

        private void btnSave_Click(object sender, EventArgs e)
        {
            if (this.pbImage.Image == null)
            {
                var regKey = Registry.CurrentUser;
                regKey = regKey.CreateSubKey("Control Panel\\Desktop");
                regKey.SetValue("Wallpaper", "");
                regKey.SetValue("TileWallpaper", "0");
                regKey.SetValue("WallpaperStyle", 2);

                SystemParametersInfo(SPI_SETDESKWALLPAPAER, 0, "", 0);
            }
            else
            {
                var FileType = FullPath.ToUpper().Split(new Char[] { '.' });
                System.Array.Reverse(FileType);
                var FileFullName = FullPath.Split(new Char[] { '\\' });
                System.Array.Reverse(FileFullName);
                var FileName = FileFullName[0].Split(new Char[] { '.' });
                if (FileType[0] != "BMP")
                {
                    var f = new FileInfo(Application.StartupPath + @"\img\" + FileName[0] + ".bmp");
                    var NewFileName = "";
                    var i = 0;
                    var b = true;
                    if (f.Exists)
                    {
                        while (b)
                        {
                            var fe = new FileInfo(Application.StartupPath + @"\img\" + FileName[0] + "_" + i.ToString() + ".bmp");
                            if (!fe.Exists)
                            {
                                NewFileName = FileName[0] + "_" + i.ToString();
                                break;
                            }
                            i++;
                        }
                    }
                    else
                        NewFileName = FileName[0];

                    var NewImage = new Bitmap(Image.FromFile(FullPath));
                    NewImage.Save(Application.StartupPath + @"\img\" + NewFileName + ".bmp", System.Drawing.Imaging.ImageFormat.Bmp);

                    var regKey = Registry.CurrentUser;
                    regKey = regKey.CreateSubKey("Control Panel\\Desktop");
                    regKey.SetValue("ConvertedWallpaper", FullPath);
                    regKey.SetValue("OriginalWallpaper", Application.StartupPath + @"\img\" + NewFileName + ".bmp");
                    regKey.SetValue("Wallpaper", Application.StartupPath + @"\img\" + NewFileName + ".bmp");
                    if (this.cbPosition.Text == "바둑판")
                        regKey.SetValue("TileWallpaper", "1");
                    else
                        regKey.SetValue("TileWallpaper", "0");
                    regKey.SetValue("WallpaperStyle", StyleCheck());
                    SystemParametersInfo(SPI_SETDESKWALLPAPAER, 0, Application.StartupPath + @"\img\" + NewFileName + ".bmp", 0);
                }
                else
                {
                    var regKey = Registry.CurrentUser;
                    regKey = regKey.CreateSubKey("Control Panel\\Desktop");
                    regKey.SetValue("Wallpaper", FullPath);
                    if (this.cbPosition.Text == "바둑판")
                        regKey.SetValue("TileWallpaper", "1");
                    else
                        regKey.SetValue("TileWallpaper", "0");
                    regKey.SetValue("WallpaperStyle", StyleCheck());
                    SystemParametersInfo(SPI_SETDESKWALLPAPAER, 0, FullPath, 0);
                }
            }
            lvGetIcon();
        }
        private string StyleCheck()
        {
            if (this.cbPosition.Text == "가운데")
                return "0";
            else if (this.cbPosition.Text == "바둑판")
                return "0";
            else
                return "2";
        }
    }

    public class ExtractIcon
    {
        [DllImport("Shell32.dll")]
        private static extern int SHGetFileInfo(
            string pszPath, uint dwFileAttributes,
            out SHFILEINFO psfi, uint cbfileInfo,
            SHGFI uFlags);

        private struct SHFILEINFO
        {
            public SHFILEINFO(bool b)
            {
                hIcon = IntPtr.Zero; iIcon = 0; dwAttributes = 0; szDisplayName = ""; szTypeName = "";
            }
            public IntPtr hIcon;
            public int iIcon;
            public uint dwAttributes;
            public string szDisplayName;
            public string szTypeName;
        };

        private enum SHGFI
        {
            SmallIcon = 0x00000001,
            OpenIcon = 0x00000002,
            LargeIcon = 0x00000000,
            Icon = 0x00000100,
            DisplayName = 0x00000200,
            Typename = 0x00000400,
            SysIconIndex = 0x00004000,
            LinkOverlay = 0x00008000,
            UseFileAttributes = 0x00000010
        }

        public static Icon GetIcon(string strPath, bool bSmall, bool bOpen)
        {
            SHFILEINFO info = new SHFILEINFO(true);
            int cbFileInfo = Marshal.SizeOf(info);
            SHGFI flags;
            if (bSmall)
                flags = SHGFI.Icon | SHGFI.SmallIcon;
            else
                flags = SHGFI.Icon | SHGFI.LargeIcon;
            if (bOpen) flags = flags | SHGFI.OpenIcon;

            SHGetFileInfo(strPath, 256, out info, (uint)cbFileInfo, flags);

            return Icon.FromHandle(info.hIcon);
        }
    }
}
